function Wrot = ICA_optimization_via_rotation(W)
% Iteratively rotates pairs of rows of the input matrix W to maximize negentropy,
% until no further rotation can improve the negentropy of the rotated matrix

max_num_iterations = 100; % maximum number of possible iterations
Wrot = W;
for q = 1:max_num_iterations
    
    fprintf('Iteration %d\n', q); 
    
    % computess negentropy as a function of rotation for all pairs
    [negentropy_vs_rotation_allpairs, rotation_angles, pairs] = negentropy_vs_rotation(Wrot);
    
    % loop through each pair applying the optimal rotation
    n_pairs = size(pairs,1);
    best_rotation = nan(1,n_pairs);
    for i = 1:n_pairs
        
        % find best rotation angle
        [~,xi] = max(negentropy_vs_rotation_allpairs(:,i));
        best_rotation(i) = rotation_angles(xi);
        
        % apply the best rotation
        Wrot(pairs(i,:),:) = rotMat2D(best_rotation(i))*Wrot(pairs(i,:),:);
        
    end
    
    % stop if all rotations are zero
    if all(abs(best_rotation - 0) < 1e-6) % if no rotation
        break;
    end
    
end
