%% Create the source matrix

% read in the audio for three sounds
[cello, sr] = audioread('cello.wav');
speech = audioread('speech.wav');
typing = audioread('typing.wav');

% keep the first 4 seconds
cello = cello(1:sr*4);
speech = speech(1:4*sr);
typing = typing(1:4*sr);

% concatenate the sources as rows of a matrix
% 3 sources x number of time-points
sources = [cello'; speech'; typing'];

% plot the sources
plot((1:size(sources,2))/sr, sources');
xlabel('Time (s)'); ylabel('Waveform Amplitude');
legend('Cello','Speech','Typing');

% listen to the sources
% play_sound(cello,sr);
% play_sound(speech,sr);
% play_sound(typing,sr);

%% Create the microphone "recordings"

% mix together the sources with different weights
% create three "microphone recordings"
% 5 mics x number of time-points
microphone_weights = [...
    1.0 0.5 0.3;
    0.2 0.3 2.0;
    0.3 1.2 1.1;
    0.1 1.1 2.3;
    1.0 1.0 1.0;
    ];
microphone_audio = microphone_weights * sources;

% list to one of the microphone recordings
play_sound(microphone_audio(1,:)',sr);

%% Project microphone matrix onto the top 3 principal components

n_pca_components_to_keep = 3;

% singular value decomposition
[U,S,V] = svd(microphone_audio, 'econ');

% sources correspond to the "V" matrix, keep the top 3 components
PCA_sources = V(:,1:n_pca_components_to_keep)';

% scale PCA sources to have unit variance
PCA_sources = zscore(PCA_sources')';

% plot the inferred PCA "sources"
plot((1:size(PCA_sources,2))/sr, PCA_sources');
xlabel('Time (s)'); ylabel('Waveform Amplitude');
legend('PCA Component 1','PCA Component 2','PCA Component 3');

% play one of the pca sources
play_sound(PCA_sources(1,:)',sr);
% play_sound(PCA_sources(2,:)',sr);
% play_sound(PCA_sources(3,:)',sr);

%% ** Do this yourself ** Calculate negentropy for different rotations for all pairs of principal components

% script which computes and plots negentropy for all pairs of components
% as a function of their rotation
negentropy_vs_rotation(PCA_sources);

%% ** Do this yourslef ** Iteratively rotate ICA components to maximize negentropy

% uses the above script to iteratively rotate all pairs of components
% so as to maximize their negentropy; stops when no further rotation
% will improve the negentropy
ICA_sources = ICA_optimization_via_rotation(PCA_sources);

% listen to the inferred sources
play_sound(ICA_sources(1,:)',sr);
% play_sound(ICA_sources(2,:)',sr);
% play_sound(ICA_sources(3,:)',sr);